/*
 * server
 */
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <string.h>
#include <errno.h>
#include <signal.h>
#include <byteswap.h>

#include "xmalloc.h"
#include "log.h"
#include "list.h"
#include "tree.h"
#include "prioq.h"
#include "buffer.h"
#include "sock-unix.h"
#include "io.h"
#include "timeout.h"
#include "pi-server.h"
#include "signal-event.h"

static int master;
static struct io_context *io;
static buffer_t input_buffer;
static buffer_t output_buffer;

void handle_signals(int signo, void *opaque)
{
	switch (signo) {
	case SIGINT:
	case SIGTERM:
		info("exiting by signal %d", signo);
		io_loop_stop(io);
		break;
	default:
		info("unhanlded signal %d", signo);
		break;
	}
}

void signal_event(int fd, short events, void *opaque)
{
	if (events & EVENT_ERROR)
		error("Ooops signal fd has error!");

	if (events & EVENT_READ)
		sigevent_run();
}

int prepare_inputs_cb(void)
{
	u_int8_t datum;
	u_int16_t value_int;
	union {
		u_int32_t u32;
		float real;
	} v, a, b;

	debug("prepare inputs callback");

	return 0;
}

void prepare_inputs_end(void)
{
	u_int8_t datum;
	union {
		u_int32_t u32;
		float real;
	} a, b;

	debug("prepare inputs end callback");

}

int prepare_outputs_cb(void)
{
	union {
		u_int32_t u32;
		float real;
	} a, b;

	debug("prepare outputs callback");

	return 0;
}

void  prepare_outputs_end(void)
{
	debug("prepare outputs end callback");
}

int main(int argc, char *argv[])
{
	int i, res;
	unsigned char *data;
	size_t inputs_len, outputs_len;
	void *inputs, *outputs;
	struct pi_server_callbacks callbacks;

	log_init(argv[0], LOG_LEVEL_DEBUG, 0, LOG_FACILITY_DAEMON, NULL);

	if (argc > 2) {
		if (strcmp(argv[1], "master") == 0)
			master = 1;
		else if (strcmp(argv[1], "slave") == 0)
			master = 0;
		else {
			printf("%s: Please provide master or slave mode\n", argv[0]);
			exit(-1);
		}
	} else {
		printf("Usage: %s [master|slave] socketfile\n", argv[0]);
		exit(-1);
	}

	sigevent_ignore(SIGPIPE);
	sigevent_add(SIGINT, handle_signals, NULL, 0);
	sigevent_add(SIGTERM, handle_signals, NULL, 0);
	sigevent_init();

	inputs_len = 256;
	inputs = xmalloc(inputs_len);
	outputs_len = 256;
	outputs = xmalloc(outputs_len);

	input_buffer = buffer_create_static(inputs, 256);
	output_buffer = buffer_create_static(outputs, 256);
	buffer_advance(input_buffer, 256);
	buffer_advance(output_buffer, 256);

/*	data = inputs;
	for (i = 0; i < inputs_len; i++)
		data[i] = i;
	data = outputs;
	for (i = 0; i < outputs_len; i++)
		data[i] = i;
*/
	timeout_init();
	io = io_new();
	io_add(io, sigevent_fd(), EVENT_READ, signal_event, NULL);

	res = pi_server_init(io, argv[2],
			inputs, inputs_len, outputs, outputs_len, 0);
	if (res != 0)
		error("can't initialize server");
	memset(&callbacks, 0, sizeof(callbacks));
	callbacks.prepare_read_callback = prepare_inputs_cb;
	callbacks.prepare_write_callback = prepare_outputs_cb;
	callbacks.end_read_callback = prepare_inputs_end;
	callbacks.end_write_callback = prepare_outputs_end;
	pi_server_register_callbacks(&callbacks, NULL);

	io_loop_run(io);

	sigevent_deinit();
	pi_server_deinit();
	timeout_deinit();
	buffer_free(input_buffer);
	buffer_free(output_buffer);
	io_free(&io);
	xfree(inputs);
	xfree(outputs);
	return 0;
}
